-- Put this first to make sure multiple statements are processed

CREATE TABLE quick_testx (
    test_id     INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
    test_val    TEXT
);

CREATE TABLE quick_test (
    test_id     INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
    test_val    TEXT
);
